﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Resources;
using System.Text;
using System.Windows.Forms;

using MindFusion.Diagramming;
using MindFusion.Drawing;

namespace LogicModel
{
    public partial class Form1 : Form
    {
        //right-click menu
        ContextMenuStrip cMenu;
        //keeps the cell the user has clicked on
        TableNode.Cell clickedCell = null;
     
        public Form1()
        {
            InitializeComponent();

            //identify the start of the text to appear as a hyperlink in the label
            int linkStart = linkLabel1.Text.IndexOf("https");
            if(linkStart > -1)
            this.linkLabel1.Links.Add(linkStart, linkLabel1.Text.Length - 1, "www.mindfusion.eu/flowchart-net.html");
           

            //tracks when the user clicks on a table cell
            diagram1.CellClicked += new EventHandler<CellEventArgs>(diagram1_CellClicked);

            //styling the links
            diagram1.LinkShape = LinkShape.Cascading;
            diagram1.LinkCascadeOrientation = MindFusion.Diagramming.Orientation.Horizontal;
            diagram1.LinkHeadShape = ArrowHeads.Triangle;
            diagram1.LinkHeadShapeSize = 4;

            //create a theme to apply additional link styling
            Theme theme = new Theme();
            DiagramLinkStyle style = new DiagramLinkStyle();
            style.Brush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(192, 192, 192));
            style.Stroke = new MindFusion.Drawing.SolidBrush(Color.FromArgb(80, 80, 80));
            theme.RegisterStyle(typeof(DiagramLink), style);
            diagram1.Theme = theme;

            //initialize the context menu items
            cMenu = new ContextMenuStrip();
            cMenu.Items.Add("Add row");
            cMenu.Items.Add("Delete row");
            cMenu.ItemClicked += new ToolStripItemClickedEventHandler(cMenu_ItemClicked);

            //the table with resources
            TableNode tbResources = new TableNode();
            tbResources.RowHeight = 20;
            tbResources.RedimTable(2, 4);
            tbResources.Caption = "Investment";
            tbResources.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(192, 192, 192));
            tbResources.CaptionHeight = 25;
            tbResources.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.White);
            tbResources.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(247, 150, 70));
            tbResources[1, 0].Text = "People";
            tbResources[1, 1].Text = "Money";
            tbResources[1, 2].Text = "Technology";
            tbResources[1, 3].Text = "Space";

            var assembly = Assembly.GetExecutingAssembly();
            var imageNames = new string[] { "orange", "red", "blue", "green" };
            var images = imageNames.
                Select(name =>
                {
                    var imagePath = $"LogicModel.Resources.{name}.png";
                    using (var stream = assembly.GetManifestResourceStream(imagePath))
                        return Image.FromStream(stream); 
                }).
                ToArray();

            tbResources.Tag = images[0];
            //set an orange bullet before each label
            for (int i = 0; i < tbResources.Rows.Count; i++)
            {
                tbResources[0, i].Image = images[0];
                //save the row index as a tag for cells with text
                tbResources[1, i].Tag = i;
            }

            tbResources.Brush = new LinearGradientBrush(Color.FromArgb(252, 213, 181), Color.FromArgb(248, 234, 223));
            tbResources.Bounds = new RectangleF(0, 0, 50, 50);
            nodeListView1.AddNode(tbResources);

            //table for the input
            TableNode tbInput = new TableNode();
            tbInput.RowHeight = 20;
            tbInput.RedimTable(2, 11);
            tbInput.Caption = "Resources";
            tbInput.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(192, 192, 192));
            tbInput.CaptionHeight = 25;
            tbInput.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.White);
            tbInput.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(248, 104, 70));
            tbInput[1, 0].Text = "Teachers";
            tbInput[1, 1].Text = "Consultants";
            tbInput[1, 2].Text = "Experts";
            tbInput[1, 3].Text = "Donations";
            tbInput[1, 4].Text = "Fees";
            tbInput[1, 5].Text = "Software";
            tbInput[1, 6].Text = "Hardware";
            tbInput[1, 7].Text = "Online Communities";
            tbInput[1, 8].Text = "Office Rooms";
            tbInput[1, 9].Text = "Rented Space";
            tbInput[1, 10].Text = "Campus Room";
            //keep the bullet location as a tag
            tbInput.Tag = images[1];

            for (int i = 0; i < tbInput.Rows.Count; i++)
            {
                tbInput[0, i].Image = images[1];
                //save the row index of the cell as a tag for cells with text
                tbInput[1, i].Tag = i;
            }

            tbInput.Brush = new LinearGradientBrush(Color.FromArgb(239, 164, 147), Color.FromArgb(248, 221, 215));
            tbInput.Bounds = new RectangleF(0, 0, 50, 50);
            nodeListView1.AddNode(tbInput);

            //table for the activities
            TableNode tbActivities = new TableNode();
            tbActivities.RowHeight = 20;
            tbActivities.RedimTable(2, 6);
            tbActivities.Caption = "Activities";
            tbActivities.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(192, 192, 192));
            tbActivities.CaptionHeight = 25;
            tbActivities.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.White);
            tbActivities.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(94, 162, 255));
            tbActivities[1, 0].Text = "Training Classes";
            tbActivities[1, 1].Text = "Community Meetings";
            tbActivities[1, 2].Text = "Published Materials";
            tbActivities[1, 3].Text = "Reports";
            tbActivities[1, 4].Text = "Support Events";
            tbActivities[1, 5].Text = "Conferences";
            //keep the bullet location as a tag
            tbActivities.Tag = images[2];

            for (int i = 0; i < tbActivities.Rows.Count; i++)
            {
                tbActivities[0, i].Image = images[2];
                //save the row index of the cell as a tag for cells with text
                tbActivities[1, i].Tag = i;
            }

            tbActivities.Brush = new LinearGradientBrush(Color.FromArgb(153, 193, 247), Color.FromArgb(208, 223, 245));
            tbActivities.Bounds = new RectangleF(0, 0, 50, 50);
            nodeListView1.AddNode(tbActivities);

            //table for the students
            TableNode tbStudents = new TableNode();
            tbStudents.RowHeight = 20;
            tbStudents.RedimTable(2, 5);
            tbStudents.Caption = "Participants";
            tbStudents.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(192, 192, 192));
            tbStudents.CaptionHeight = 25;
            tbStudents.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.White);
            tbStudents.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(94, 162, 255));
            tbStudents[1, 0].Text = "Larger Audience";
            tbStudents[1, 1].Text = "Greater Diversity";
            tbStudents[1, 2].Text = "Longer Courses";
            tbStudents[1, 3].Text = "Higher Satisfaction";
            tbStudents[1, 4].Text = "Bigger Involvement";
            //keep the bullet location as a tag
            tbStudents.Tag = images[3];

            for (int i = 0; i < tbStudents.Rows.Count; i++)
            {
                tbStudents[0, i].Image = images[3];
                //save the row index of the cell as a tag for cells with text
                tbStudents[1, i].Tag = i;
            }

            tbStudents.Brush = new LinearGradientBrush(Color.FromArgb(153, 193, 247), Color.FromArgb(208, 223, 245));
            tbStudents.Bounds = new RectangleF(0, 0, 50, 50);
            nodeListView1.AddNode(tbStudents);

            //table for the participants
            TableNode tbParticipants = new TableNode();
            tbParticipants.RowHeight = 20;
            tbParticipants.RedimTable(2, 5);
            tbParticipants.Caption = "Students";
            tbParticipants.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(192, 192, 192));
            tbParticipants.CaptionHeight = 25;
            tbParticipants.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.White);
            tbParticipants.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(153, 176, 105));
            tbParticipants[1, 0].Text = "Higher Grades"; 
            tbParticipants[1, 1].Text = "Self-Development";
            tbParticipants[1, 2].Text = "Better Competiveness";
            tbParticipants[1, 3].Text = "Self Confidence";
            tbParticipants[1, 4].Text = "Professional Realization";
            //keep the bullet location as a tag
            tbParticipants.Tag = images[3];

            for (int i = 0; i < tbParticipants.Rows.Count; i++)
            {
                tbParticipants[0, i].Image = images[3];
                //save the row index of the cell as a tag for cells with text
                tbParticipants[1, i].Tag = i;
            }

            tbParticipants.Brush = new LinearGradientBrush(Color.FromArgb(179, 206, 122), Color.FromArgb(217, 221, 210));
            tbParticipants.Bounds = new RectangleF(0, 0, 50, 50);
            nodeListView1.AddNode(tbParticipants);

            //table for the teachers
            TableNode tbTeachers = new TableNode();
            tbTeachers.RowHeight = 20;
            tbTeachers.RedimTable(2, 5);
            tbTeachers.Caption = "Teachers";
            tbTeachers.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(192, 192, 192));
            tbTeachers.CaptionHeight = 25;
            tbTeachers.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.White);
            tbTeachers.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(153, 176, 105));
            tbTeachers[1, 0].Text = "Qalification";
            tbTeachers[1, 1].Text = "Experience";
            tbTeachers[1, 2].Text = "Organizational Skills";
            tbTeachers[1, 3].Text = "Extra Income";
            tbTeachers[1, 4].Text = "Connections";
            tbTeachers[1, 4].Text = "Community Work";
            //keep the bullet location as a tag
            tbTeachers.Tag = images[3];

            for (int i = 0; i < tbTeachers.Rows.Count; i++)
            {
                tbTeachers[0, i].Image = images[3];
                //save the row index of the cell as a tag for cells with text
                tbTeachers[1, i].Tag = i;
            }

            tbTeachers.Brush = new LinearGradientBrush(Color.FromArgb(179, 206, 122), Color.FromArgb(217, 221, 210));
            tbTeachers.Bounds = new RectangleF(0, 0, 50, 50);
            nodeListView1.AddNode(tbTeachers);


            //the first container for the Input
            ContainerNode cInput = diagram1.Factory.CreateContainerNode(0, 0, 20, 20);
            cInput.Tag = 0;
            cInput.Caption = "Input";
            cInput.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.White);
            cInput.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(250, 182, 125));
            cInput.Brush = new LinearGradientBrush(Color.FromArgb(250, 230, 213), Color.FromArgb(247, 239, 231));
            cInput.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(80, 80, 80));
            cInput.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(80, 80, 80));
            cInput.CaptionHeight = 10;
            cInput.AllowAddChildren = true;
            //do not allow the container to change size or be folded
            cInput.AutoGrow = false;
            cInput.AutoShrink = false;
            cInput.Foldable = false;

            //the second container for the output
            ContainerNode cOutput = diagram1.Factory.CreateContainerNode(0, 0, 20, 20);
            cOutput.Tag = 1;
            cOutput.Caption = "Output";
            cOutput.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(80, 80, 80));
            cOutput.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(80, 80, 80));
            cOutput.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(94, 162, 255));
            cOutput.Brush = new LinearGradientBrush(Color.FromArgb(193, 217, 249), Color.FromArgb(233, 241, 252));
            cOutput.CaptionHeight = 10;
            cOutput.AllowAddChildren = true;
            //do not allow the container to change size or be folded
            cOutput.AutoShrink = false;
            cOutput.AutoGrow = false;
            cOutput.Foldable = false;

            //the last container
            ContainerNode cOutcomes = diagram1.Factory.CreateContainerNode(0, 0, 20, 20);
            cOutcomes.Tag = 2;
            cOutcomes.Caption = "Outcomes";
            cOutcomes.Pen = new MindFusion.Drawing.Pen(Color.FromArgb(80, 80, 80));
            cOutcomes.CaptionBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(80, 80, 80));
            cOutcomes.CaptionBackBrush = new MindFusion.Drawing.SolidBrush(Color.FromArgb(168, 192, 120));
            cOutcomes.Brush = new LinearGradientBrush(Color.FromArgb(209, 224, 180), Color.FromArgb(235, 238, 229));
            cOutcomes.CaptionHeight = 10;
            cOutcomes.AllowAddChildren = true;
            //do not allow the container to change size or be folded
            cOutcomes.AutoGrow = false;
            cOutcomes.AutoShrink = false;
            cOutcomes.Foldable = false;

            resizeContainers();

        }

        //handles the cellClicked event
        void diagram1_CellClicked(object sender, CellEventArgs e)
        {
            //if the user has clicked with the right mouse button
            if(e.MouseButton == MouseButton.Right)
            {
                //we should keep track of the clicked cell
                clickedCell = e.Cell;
                //and show the context menu
                cMenu.Show(Cursor.Position);
                
            }
            
        }

        //resize the containers according to the current size of the diagramView
        private void resizeContainers()
        {
            //convert the size of the rectangle from client measure units to diagram units
            RectangleF viewRect = diagramView1.ClientToDoc(diagramView1.ClientRectangle);

            //identify the containers
            foreach (DiagramNode node in diagram1.Nodes)
            {
                ContainerNode c = node as ContainerNode;

                if (c != null)
                {
                    //get the index that tells us if this is the first, second or third container
                    int index = (int)c.Tag;
                    //relocate the container rectangle based on its index
                    c.Bounds = new RectangleF(viewRect.Left + viewRect.Width * (0.02F + index * 0.34F) ,
                viewRect.Top + viewRect.Height * 0.02F, viewRect.Width * 0.27F, viewRect.Height * 0.96F);
                }
            }

        }

        //handles the ItemClicked event for the context menu
        void cMenu_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            if (clickedCell != null)
            {
                TableNode tNode = clickedCell.Table;

                //if the user has selected to add a row
                if (e.ClickedItem.Text.Equals("Add row"))
                {
                    //add a row
                    tNode.AddRow();
                    //assign the image URL kept as a tag to the first cell in the new row
                    tNode[0, tNode.Rows.Count - 1].Image =
                        tNode.Tag as Image;
                    //type in some dummy text in the text cell
                    tNode[1, tNode.Rows.Count - 1].Text = "[edit text]";               
                }
                else if (e.ClickedItem.Text.Equals("Delete row"))
                {
                    cMenu.Close();

                    //display a warning, which shows which row is about to be deleted
                    string message = "Do you want to delete row " + clickedCell.Text + "?";
                    string caption = "Confirm Delete";
                    MessageBoxButtons buttons = MessageBoxButtons.YesNo;
                    DialogResult result;

                    // Displays the MessageBox.
                    result = MessageBox.Show(message, caption, buttons);
                    //if the user has decided to delete the row
                    if (result == System.Windows.Forms.DialogResult.Yes)
                    {
                                        
                        //get the index of the row that is to be deleted
                        int rowIndex = (int)clickedCell.Tag;                        
                        tNode.DeleteRow(rowIndex);                        
                        
                    }                

                }

                //adjust the node size
                resizeNode(tNode);
                //arrange the table tags
                arrangeTags(tNode);

             }
        }

        //assigns to each row in the specified table a tag that specifies its row index
        private void arrangeTags(TableNode tNode)
        {
            for (int i = 0; i < tNode.RowCount; i++)
                tNode[1, i].Tag = i;
        }


        //adjusts the size of the specified TableNode so that it fits all its text
        private void resizeNode(TableNode tNode)
        {
            tNode.ResizeToFitText(false);
            TableNode.Column col = tNode.Columns[0];
            col.Width = 5;

            tNode.Bounds = new RectangleF(tNode.Bounds.Location, new SizeF(tNode.Bounds.Width + 5,
                tNode.Bounds.Height));

           
        }

        //raised when the user has dropped a new TableNode
        private void diagram1_NodeCreated(object sender, NodeEventArgs e)
        {            
                TableNode tNode = e.Node as TableNode;

                if (tNode != null)
                {
                    //adjust the node size
                    resizeNode(tNode);
                    //arrange the tags for the table cells
                    arrangeTags(tNode);
                   
                }

            
            //find out the container onto which the node was dropped, if any
            foreach (DiagramNode diagramNode in diagram1.Nodes)
            {
                if (diagramNode.ContainsPoint(e.Node.Bounds.Location))
                {
                    ContainerNode container = diagramNode as ContainerNode;
                    if (container != null)
                    {                       
                        //add the new node to its container
                         container.Add(tNode);
                       
                        break;
                    }

                }
            }
        }

       
        //raised when the size of the ClientRectangle has changed.
        private void diagramView1_ClientSizeChanged(object sender, EventArgs e)
        {
            //adjust the size of the containeres
            resizeContainers();
            //rearrange the links
            diagram1.RouteAllLinks();
        }

        //raised when the user draws a link between two tableNode-s.
        private void diagram1_LinkCreated(object sender, LinkEventArgs e)
        {
            //the link should have 3 segments
            e.Link.SegmentCount = 3;
        }       
    }
}
